function APC=cl_bblfcn(origRegX,origRegY,RegProperty,RegSigmatheta,avgsec,parameter,deltasigma,smoothing,plotyes,apccontours,apcaxis,denscontours);

%
% CL_BBLFCN  Welcome to the functional form of the BBL calculator.  This
% code was written to compute Accumulated Property Change (APC) sections
% without using the full-blown GUI.  The program assumes that instead of 
% supplying CTD casts, as in the GUI version, you will provide regularly
% gridded fields of X, Y, potential temperature/salinity/spiciness, and
% potential density. The output array, APC, is the accumulated property 
% change forced onto the same regular grid that the user supplied with the 
% input data.
%
% Inputs: 
%   origRegX      A regularly gridded array (Matlab meshgrid format) of the 
%                 x-position of the property & density data (km)
%   origRegY      A regularly gridded array (Matlab meshgrid format) of the  
%                 y-position of the property & density data (m)
%   RegProperty   A regularly gridded array of potential temperature (or 
%                 salinity or spiciness depending on what you want to 
%                 "accumulate") in the Matlab meshgrid format
%   RegSigmatheta A regularly gridded array of potential density in the
%                 Matlab meshgrid format ** please use sigma-theta minus
%                 1000 kg/m^3; i.e. data in the range 23-29, instead of
%                 1026-1029...
%   avgsec        Bottom profile - a two column array of values - first 
%                 column is the x-distance (km) and the second column is
%                 the depth (m).  Depth should be given as negative numbers.
%   parameter     Use 1 for theta, 2 for salinity, and 3 for spiciness
%                 Only affects labeling
%   deltasigma    Interpolation step.  Smaller number means more accuracy
%                 at the cost of more computation time. 0.05 is typical. 
%   smoothing     how much the data is smoothed when using ppzgrid.  A
%                 value of 2 is fairly typical.
%   plotyes       0 or 1.  0 will suppress plots.  1 will show plots.
%   apccontours   these are the contours overlaid on the final plot.  Use
%                 array of values - see example below
%   apcaxis       An array of two numbers, such as [0 2], which will
%                 determine the color range of the final accumulated
%                 property change plot
%   denscontours  these are the density contours overlaid in magenta on
%                 the final plot.  Use array of values- see example below
%
% Example: Middle Atlantic Bight test case - SEEP February 1984 section
%   APC=cl_bblfcn(origRegX,origRegY,RegTheta,RegSigmatheta,avgsec,1,0.05,2,1,[0  0.1  0.2  0.3  0.4  0.5  0.6  0.8  1  1.5  2],[0 2],[25  25.5  26  26.5  27  27.5]);
%
% Given input arrays origRegX, origRegY, RegTheta, RegSigmatheta, and avgsec,
% the program will compute accumulated POTENTIAL TEMPERATURE change 
% (parameter=1), with a deltasigma step of 0.05 kg/m^3, smoothing of 2, 
% and will show three output plots.  The APC will be contoured on these
% values: 0, .1, .2, .3, .4, .5, .6, .8. 1, 1.5, 2 and the color range of the
% final output plot will be from 0 to 2 Celcius.  Potential density contours
% will be overlaid in magenta at the following values: 25  25.5  26  26.5  27
% 27.5. The output array is named APC; the X and Y posits of the final
% values are prescribed by origRegX and origRegY (the original regularly 
% gridded distance and depth arrays).
%
% Copyright (C) 2005  C. A. Linder, clinder@whoi.edu
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Version 1.2, 10Oct2005, C. A. Linder
% http://www.whoi.edu/science/PO/people/clinder/software_bbl.html
%

xbounds=min(min(origRegX)):abs((origRegX(1,2)-origRegX(1,1))):max(max(origRegX));
ybounds=fliplr(min(min(origRegY)):abs((origRegY(2,1)-origRegY(1,1))):max(max(origRegY)));
if nanmean(nanmean(RegSigmatheta)) > 100 RegSigmatheta=RegSigmatheta-1000; end;
regsigma=[min(min(RegSigmatheta))+0.01:deltasigma:max(max(RegSigmatheta))-0.01];
numsigmas=length(regsigma);

f2=figure;
for loop=1:numsigmas
  eval(sprintf('[c%d,h%d]=contour(origRegX,origRegY,RegSigmatheta,[regsigma(%d) regsigma(%d)],''k'');',loop,loop,loop,loop));
  hold on;
  % This line is critical - the program is acquiring the x-y contours.
  eval(sprintf('c%d=c%d'';',loop,loop));
end;
if plotyes==0 close(f2); end;

numgood=0;
for loop=1:numsigmas % Begin cleaning these contour lines
  % The first line indicates how long the segment is (rows).
  eval(sprintf('linedata=c%d;',loop));
  numsegrows=linedata(1,2);
  seg1=linedata(2:numsegrows+1,:);
  numsegs=1;
 
  while length(linedata) > linedata(1,2)+1  % Then we know there are multiples - loop through & grab em.
      numsegs=numsegs+1;
      linedata=linedata(numsegrows+2:end,:);
      numsegrows=linedata(1,2);
      eval(sprintf('seg%d=linedata(2:numsegrows+1,:);',numsegs));
  end;

  for subloop=1:numsegs
      eval(sprintf('maxseg=seg%d;',subloop));
      % Now we need to run some tests to see if this segment is good or
      % not.  We want to eliminate
      % a) Segments starting AND ending at surface.
      % b) Closed loops
      % c) Segments starting AND ending on bottom
      % d) Segments starting from LHS of page
      % e) Segments starting from BOTTOM of page
      goodseg=0; % Assume the worst!
      if (maxseg(1,2) > (5+min(origRegY(:,1)))) & (maxseg(1,2) ~= maxseg(end,2)) % First eliminate any contours beginning within 5m of BOTTOM OF PLOT and closed contours
          % Let's try a new if statement... we want only those contours which are close to the bottom... so, let's find out
          % what the interpolated bottom depth is for a given x-loc of contour, and then if it's within 5m of that number, include it.
          realorigboty=interp1(avgsec(:,1),avgsec(:,2),maxseg(1,1));
          realtermboty=interp1(avgsec(:,1),avgsec(:,2),maxseg(end,1));
          % Now, check that one end is > 10m from bottom
          origdiff=abs(realorigboty - maxseg(1,2));  % This is the difference between the start of the segment and the bottom
          termdiff=abs(realtermboty - maxseg(end,2));  % This is the difference between the end of the segment and the bottom
          if origdiff < 10
              if termdiff < 10
                  % BAD - it originates and terminates on the bottom!
                  goodseg=0;
              else % This is the best case - segment originates within 10m, and terminates > 10m
                  goodseg=1;    
              end;
          elseif termdiff < 10
              maxseg=flipud(maxseg);
              goodseg=1;
          else
              % BAD - both origination and terimination > 10m from bottom
              goodseg=0;
          end;
          % Now we know what the orientation is, let's just kick out those
          % that terminate on the LHS or BOTTOM, we don't want those.
          if maxseg(end,2) < (5+min(origRegY(:,1))) % Terminating on bottom
              goodseg=0;
          end;
          if maxseg(end,1) < (5+min(origRegX(1,:))) % Terminating on left wall
              goodseg=0;
          end;
          if(goodseg)
              numgood=numgood+1;
              eval(sprintf('cnew%d=maxseg;',numgood));      
              if(plotyes)
                  eval(sprintf('plot(cnew%d(:,1),cnew%d(:,2),''mo-'',''markersize'',1,''linewidth'',1);',numgood,numgood));
                  eval(sprintf('plot(cnew%d(1,1),cnew%d(1,2),''g.'',''markersize'',10);',numgood,numgood));
                  eval(sprintf('plot(cnew%d(end,1),cnew%d(end,2),''r.'',''markersize'',10);',numgood,numgood));
              end;
          end;
      end;

      
  end;
end; % cleaning

if(plotyes)
    xlabel('Distance from first cast (km)');
    ylabel('Depth (m)');
    yt=get(gca,'ytick');
    set(gca,'ytick',yt);
    set(gca,'yticklabel',abs(yt));
    hold on; plot(avgsec(:,1),avgsec(:,2),'k');
    title(['Sigma-theta contours that program is going to integrate along.  Green/red dots indicate start/end points.']);
    ylim([min(ybounds) max(ybounds)+2]);
end;


if numgood~=0

% accum T/salinity/Spice along isopycnals...

for loop=1:numgood
  eval(sprintf('t%d=interp2(origRegX,origRegY,RegProperty,cnew%d(:,1),cnew%d(:,2));',loop,loop,loop));   
  eval(sprintf('inonan=find(isnan(t%d)==0);',loop));
  eval(sprintf('t%d=t%d(inonan);',loop,loop));
  eval(sprintf('temp=t%d;',loop));
  if isempty(temp)
    eval(sprintf('x%d=NaN;',loop));
    eval(sprintf('z%d=NaN;',loop));
    eval(sprintf('accumt%d=NaN;',loop));
  else
    eval(sprintf('x%d=cnew%d(inonan,1);',loop,loop));
    eval(sprintf('z%d=cnew%d(inonan,2);',loop,loop));
    eval(sprintf('accumt%d=cumsum(abs(gradient(t%d-t%d(1))));',loop,loop,loop));
  end;
end;

if(plotyes)

f4=figure;
for loop=1:numgood
  eval(sprintf('scatter(x%d,z%d,20,accumt%d,''filled'');',loop,loop,loop));
  if isempty(apcaxis)==0
    caxis(apcaxis);
  end;
  hold on;
end;
axis([min(min(origRegX)) max(max(origRegX)) min(min(origRegY)) max(max(origRegY))]); 
grid on;
plot(avgsec(:,1),avgsec(:,2),'k');
xlabel('Distance from first cast (km)');
ylabel('Depth (m)');
yt=get(gca,'ytick');
set(gca,'ytick',yt);
set(gca,'yticklabel',abs(yt));
hold on; plot(avgsec(:,1),avgsec(:,2),'k');
switch parameter
case 1
   title(['Raw (scatterplot) accumulated theta change (',setstr(176),'C)']);
case 2
   title(['Raw (scatterplot) accumulated salinity change']);
case 3
   title(['Raw (scatterplot) accumulated spiciness change (kg/m^3)']);
end;
end; % plotyes

xyzfile=[];
for loop=1:numgood
   eval(sprintf('xyzfile=[xyzfile; x%d z%d accumt%d];',loop,loop,loop));
end;

blankfile=[avgsec; avgsec(length(avgsec),1) min(avgsec(:,2))-10; min(xbounds) min(avgsec(:,2))-10; avgsec(1,:)];
save blankfile.temp blankfile -ascii

eval(sprintf('[Z,grd]=ppzinit(xyzfile,''-R%d/%d/%d/%d -I%d/%d -Mblankfile.temp'');',min(xbounds),max(xbounds),min(ybounds),max(ybounds),abs(xbounds(2)-xbounds(1)),abs(ybounds(2)-ybounds(1))));
if smoothing > 0 
    eval(sprintf('[ppz,grdout]=ppsmooth(Z,grd,''-S%d'');',smoothing));
else
    ppz=Z;
end;
APC=flipud(ppz);
[Xnew,Ynew]=meshgrid([min(xbounds):abs(xbounds(2)-xbounds(1)):max(xbounds)],[max(ybounds):-abs(ybounds(2)-ybounds(1)):min(ybounds)]);

if(plotyes)
f5=figure; subplot(211);
[c,h]=contourf(origRegX,origRegY,RegProperty,[floor(min(min(RegProperty))):1:ceil(max(max(RegProperty)))]);
for loop=1:length(h); set(h(loop),'linestyle','none'); end;
hold on;
[cbw,hbw]=contour(origRegX,origRegY,RegProperty,[0:1:50],'k');
clabel(cbw,hbw,'fontweight','bold');
[cm,hm]=contour(origRegX,origRegY,RegSigmatheta,denscontours,'m');
set(hm,'linewidth',2);
clabel(cm,hm,'color','m','fontweight','bold');
fill(blankfile(:,1),blankfile(:,2),'w');
colormap(jet);
axis([min(xbounds) max(xbounds)  min(ybounds) max(ybounds)]);
switch parameter
case 1
   title(['Theta (',setstr(176),'C)']);
case 2
   title(['Salinity']);
case 3
   title(['Spiciness (kg/m^3)']);
end;

ylabel('Depth (m)');
yt=get(gca,'ytick');
set(gca,'ytick',yt);
set(gca,'yticklabel',abs(yt));

subplot(212);
%[C,H,CF]=contourf(origRegX,origRegY,APC,apccontours); hold on; %Matlab v65

[C,H]=contourf('v6',origRegX,origRegY,APC,apccontours); hold on; %Matlab v7
caxis([min(apccontours) max(apccontours)]);

for junkloop=1:length(H) set(H(junkloop),'linestyle','none'); end;
colormap(jet);
if apccontours(1)==0
    [c,h]=contour(origRegX,origRegY,APC,apccontours(2:end),'k');
else
    [c,h]=contour(origRegX,origRegY,APC,apccontours,'k');
end;
clabel(c,h,'fontweight','bold');
hold on;
[cm,hm]=contour(origRegX,origRegY,RegSigmatheta,denscontours,'m');
set(hm,'linewidth',2);
clabel(cm,hm,'color','m','fontweight','bold'); 

hold on; fill(blankfile(:,1),blankfile(:,2),'w');
axis([min(xbounds) max(xbounds)  min(ybounds) max(ybounds)]);
switch parameter
case 1
   title(['Accumulated theta change (',setstr(176),'C)']);
case 2
   title(['Accumulated salinity change - deltasigmatheta = ',num2str(deltasigma)]);
case 3
   title(['Accumulated spiciness change (kg/m^3) - deltasigmatheta = ',num2str(deltasigma)]);
end;
xlabel(['Distance from first cast (km)']);
ylabel(['Depth (m)']);
yt=get(gca,'ytick');
set(gca,'ytick',yt);
set(gca,'yticklabel',abs(yt));
end; % plotyes

end; % if numgood=0