% cl_bblgui.m
%
% Bottom Boundary Layer calculator
%
% Welcome to the BBL GUI.  Given input casts arranged in a section, this 
% tool computes the bottom boundary layer tracer "accumulated theta 
% difference", displays it along with associated plots, and can save out
% the data. Click on the Help buttons for explanations of the options and 
% buttons.
%
% There are some restrictions regarding file format.  The program expects 
% the following format (and will crash if format not followed!)
%
% - Input file: a Matlab .mat file containing these arrays:
% One six-column array named "posits"
% 1: Station number (1 through Numstations)
% 2: Latitude degrees (- for South)
% 3: Latitude decimal minutes (- for South)
% 4: Longitude degrees (- for West)
% 5: Longitude decimal minutes (- for West)
% 6: Bottom depth of cast (meters)
%
% One array for each station named "rawprofileX" where X is the station 
% number from the posits array - each is a 3 column array
% 
% 1: Pressure (dbar)
% 2: Temperature (C)
% 3: Salinity
%
% Program options:
% - Deltasigma.  Interpolation step.  Smaller number means more computation 
% time, but more accuracy.
% - Parameter choice.  Choose to accumulate potential temperature (theta), 
% salinity, or spiciness
% - Save outfile.mat.  This resaves the range, posits, and rawprofiles, with
% one more column appended- the accumulated theta. The output filename is 
% outfile.mat.
%
% Gridding/Plotting options
% - Spacing and limits - self explanatory.  If you need help choosing these 
% limits, hit the "Data query" button
% - Smoothing - how much the data is smoothed when using ppzgrid.  A value
% of 2 is good for most oceanographic applications.  0 is no smoothing.
% - Sigma-theta contours - these are the contours overlaid on many plots.  
% Pick as many as you like.
% - Accumulated theta contours - these are the contours overlaid on the 
% final plot.
% - ATC colorbar - the color range of the final ATC plot.  Leave blank for 
% full range.
% - Data query - displays the range, depth, and sigmatheta limits of your 
% data.
% - Compute/Plot - computes the accumulated theta difference, and displays 
% selected plots.
%
% Supporting mfiles: cl_bblini.m, cl_bblplt.m, cl_bbllnd.m, cl_bblqry.m, 
% cl_bblabt.m, cl_bblcomputehelp.m, cl_bblplothelp.m, cl_bblupd.m, 
% cl_bblsave.m, sw_dist, sw_dpth, sw_ptmp, sw_dens0 (CSIRO toolbox),
% the Matlab ppzgrid package from Roger Goldsmith (not necessary).
%
% Usage: cl_bblgui
% Click on help buttons for tips.
%
% Version 1.2, 10Oct2005, C. A. Linder
% http://www.whoi.edu/science/PO/people/clinder/software_bbl.html
%

%*****Set defaults*****
opt1 = 10;   opt2 = 10;  opt3 = 230;  opt4 = 290; 
plot1 = 250;  plot2 = 10;  plot3 = 230; plot4 = 290; 
format compact;
fgui=figure;
clf;
set(fgui,'Position',[30 30 490 310]);
set(fgui,'Name','BBL GUI');
screenstats = get(gcf,'Position');
cl_bblini;
if exist(bbldirectory)==7
   cd(bbldirectory);
else
   bbldirectory=pwd;
end;
more off;

%*****Pathframe*****
frame_path = uicontrol(fgui,...
  'Style','frame',...
  'Position',[opt1 opt2+260 opt3 30]);

%*****Path*****
text_pathfil = uicontrol(fgui,...
  'Style','text',...
  'String','Path:',...
  'HorizontalAlignment','Left',...
  'Position',[(opt1+10) (opt2+270) (opt3-110) 13]);
en_pathfil = uicontrol(fgui,...
  'Style','edit',...
  'String',bbldirectory,...
  'Position',[(opt1+40) (opt2+265) 180 20],...
  'CallBack',[...
     'bbldirectory = get(en_pathfil,''String'');,',...
     'if exist(bbldirectory)==7 cd(bbldirectory); else ,',...
     'msgbox(''Directory not found.  Please reenter.'',''Error'',''Error'');,',...
     'bbldirectory=pwd; set(en_pathfil,''String'',pwd); end;']);

%*****Clear figures button commands*****
pb_clearfigs = uicontrol(fgui,...
  'Style','push',...
  'String','Clear figs',...
  'Pos',[opt1 235 55 30],...
  'CallBack',[...
    'numfigs = 0;,',...
    '[flag,killfigs] = figflag('''');,',...
    'delete(killfigs);']);

%*****Save defaults button commands*****
pb_savedefaults = uicontrol(fgui,...
  'Style','push',...
  'String','Save defaults',...
  'Pos',[opt1+60 235 80 30],...
  'CallBack','cl_bblsave');


%*****Help button commands*****
pb_help = uicontrol(fgui,...
  'Style','push',...
  'String','About',...
  'Pos',[opt1+145 235 45 30],...
  'CallBack','cl_bblabt');

%*****Exit button commands*****
pb_exit = uicontrol(fgui,...
  'Style','push',...
  'String','Exit',...
  'Position',[opt1+195 235 35 30],...
  'CallBack','close(fgui)');

%*****LHS frame*****
frame_inputfil = uicontrol(fgui,...
  'Style','frame',...
  'Position',[opt1 opt2 opt3 opt4-75]);


text_computeoptions = uicontrol(fgui,...
  'Style','text',...
  'String','BBL compute options',...
  'HorizontalAlignment','Left',...
  'Position',[20 195 150 20]);
pb_computehelp = uicontrol(fgui,...
  'Style','push',...
  'String','Help',...
  'Pos',[190 200 40 20],...
  'CallBack','cl_bblcomputehelp');

%*****Inputfile*****
text_inputfile = uicontrol(fgui,...
  'Style','text',...
  'String','Input file:',...
  'HorizontalAlignment','Left',...
  'Position',[(opt1+10) (opt2+155) (opt3-160) 20]);
en_inputfile = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(opt1+90) (opt2+160) 130 20],...
  'String',inputfile,...
  'CallBack',[...
     'inputfile = get(en_inputfile,''String'');,',...
     'if exist(strcat(inputfile,''.mat''))~=2 msgbox(''File not found.  Please reenter.'',''Error'',''Error''); end;']);

%*****Deltasigma*****
text_deltasigma = uicontrol(fgui,...
  'Style','text',...
  'String','Delta sigma:',...
  'HorizontalAlignment','Left',...
  'Position',[(opt1+10) (opt2+130) (opt3-160) 20]);
en_deltasigma = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(opt1+90) (opt2+135) 130 20],...
  'String',num2str(deltasigma),...
  'CallBack','deltasigma = str2num(get(en_deltasigma,''String''));');

%*****Parameter*****
text_param = uicontrol(fgui,...
  'Style','text',...
  'String','Choose parameter:',...
  'HorizontalAlignment','Center',...
  'Position',[(opt1+10) (opt2+100) 210 20]);
rb_theta = uicontrol(gcf,...
  'Style','radio',...
  'String','Accumulated theta change',...
  'Position',[(opt1+10) (opt2+85) (opt3-20) 20],...
  'Value',rb_theta,...
  'CallBack',[...
    'set(rb_theta,''Value'',1),',...
    'set(rb_salinity,''Value'',0),',...
    'set(rb_spice,''Value'',0),',...
    'parameter = 1;']);
rb_salinity = uicontrol(gcf,...
  'Style','radio',...
  'String','Accumulated salinity change',...
  'Position',[(opt1+10) (opt2+65) (opt3-20) 20],...
  'Value',rb_salinity,...
  'CallBack',[...
    'set(rb_theta,''Value'',0),',...
    'set(rb_salinity,''Value'',1),',...
    'set(rb_spice,''Value'',0),',...
    'parameter = 2;']);
rb_spice = uicontrol(gcf,...
  'Style','radio',...
  'String','Accumulated spiciness change',...
  'Position',[(opt1+10) (opt2+45) (opt3-20) 20],...
  'Value',rb_spice,...
  'CallBack',[...
     'set(rb_theta,''Value'',0),',...
     'set(rb_salinity,''Value'',0),',...
     'set(rb_spice,''Value'',1),',...
     'parameter = 3;']);

%*****Save out?*****
cb_saveout = uicontrol(fgui,...
  'Style','checkbox',...
  'String','Save outfile.mat?',...
  'Position',[(opt1+10) 20 (opt3-20) 20],...
  'Value',saveout,...
  'CallBack',[...
    'saveout = 0;,',...
    'if get(cb_saveout,''Value'')==1,',...
      'saveout = 1;,',...
    'end']);

%*****Plot options*****
frame_plot = uicontrol(fgui,...
  'Style','frame',...
  'Position',[plot1 plot2 plot3 plot4]);
text_plot = uicontrol(fgui,...
  'Style','text',...
  'HorizontalAlignment','Left',...
  'String','Gridding/Plotting options',...
  'Position',[(plot1+10) (plot4-20) 150 20]);
pb_plothelp = uicontrol(fgui,...
  'Style','push',...
  'String','Help',...
  'Pos',[(plot1+180) (plot4-15) 40 20],...
  'CallBack','cl_bblplothelp');



%*****Gridding limits*****
text_DelX = uicontrol(gcf,...
  'Style','text',...
  'String','X spacing (km): ',...
  'Position',[(plot1+50) (plot4-45) (plot3-70) 20]);
en_DelX = uicontrol(gcf,...
  'Style','edit',...
  'Pos',[(plot1+170) (plot4-40) 50 20],...
  'String',num2str(DelX),...
  'CallBack',[...
    'DelX = str2num(get(en_DelX,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxx,''String'',num2str(MaxX));']);
text_minx = uicontrol(gcf,...
  'Style','text',...
  'String','MinX:',...
  'Pos',[(plot1+10) (plot4-65) 40 20]);
en_minx = uicontrol(gcf,...
  'Style','edit',...
  'Pos',[(plot1+60) (plot4-60) 50 20],...
  'String',num2str(MinX),...
  'CallBack',[...
    'MinX = str2num(get(en_minx,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxx,''String'',num2str(MaxX));']);
text_maxx = uicontrol(gcf,... 
  'Style','text',... 
  'String','MaxX:',... 
  'Pos',[(plot1+120) (plot4-65) 40 20]); 
en_maxx = uicontrol(gcf,... 
  'Style','edit',... 
  'Pos',[(plot1+170) (plot4-60) 50 20],...
  'String',num2str(MaxX),...
  'CallBack',[...
    'MaxX = str2num(get(en_maxx,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxx,''String'',num2str(MaxX));']);
text_DelY = uicontrol(gcf,...
  'Style','text',...
  'String','Y spacing (m): ',...
  'Position',[(plot1+50) (plot4-90) (plot3-70) 20]);
en_DelY = uicontrol(gcf,...
  'Style','edit',...
  'Pos',[(plot1+170) (plot4-85) 50 20],...
  'String',num2str(DelY),...
  'CallBack',[...
    'DelY = str2num(get(en_DelY,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxy,''String'',num2str(MaxY));']);
text_miny = uicontrol(gcf,...
  'Style','text',...
  'String','MinY:',...
  'Pos',[(plot1+10) (plot4-110) 40 20]);
en_miny = uicontrol(gcf,...
  'Style','edit',...
  'Pos',[(plot1+60) (plot4-105) 50 20],...
  'String',num2str(MinY),...
  'CallBack',[...
    'MinY = str2num(get(en_miny,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxy,''String'',num2str(MaxY));']);
text_maxy = uicontrol(gcf,... 
  'Style','text',... 
  'String','MaxY:',... 
  'Pos',[(plot1+120) (plot4-110) 40 20]); 
en_maxy = uicontrol(gcf,... 
  'Style','edit',... 
  'Pos',[(plot1+170) (plot4-105) 50 20],...
  'String',num2str(MaxY),...
  'CallBack',[...
    'MaxY = str2num(get(en_maxy,''String''));,',...
    'cl_bblupd;,',...
    'set(en_maxy,''String'',num2str(MaxY));']);

%*****Smoothing*****
text_smoothing = uicontrol(fgui,...
  'Style','text',...
  'String','Smoothing:',...
  'HorizontalAlignment','Left',...
  'Position',[(plot1+10) (plot4-135) (plot3-160) 20]);
en_smoothing = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(plot1+90) (plot4-130) 130 20],...
  'String',num2str(smoothing),...
  'CallBack','smoothing = str2num(get(en_smoothing,''String''));');

%*****Sigmathetacontours*****
text_sigmathetacontours = uicontrol(fgui,...
  'Style','text',...
  'String','Sigma-theta (magenta) contours:',...
  'HorizontalAlignment','Center',...
  'Position',[(plot1+10) (plot4-155) (plot3-20) 20]);
en_sigmathetacontours = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(plot1+10) (plot4-170) (plot3-20) 20],...
  'String',num2str(sigmathetacontours),...
  'CallBack','sigmathetacontours = str2num(get(en_sigmathetacontours,''String''));');

%*****Accthetacontours*****
text_accthetacontours = uicontrol(fgui,...
  'Style','text',...
  'String','Accumulated theta (black) contours:',...
  'HorizontalAlignment','Center',...
  'Position',[(plot1+10) (plot4-195) (plot3-20) 20]);
en_accthetacontours = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(plot1+10) (plot4-210) (plot3-20) 20],...
  'String',num2str(accumthetacontours),...
  'CallBack','accumthetacontours = str2num(get(en_accthetacontours,''String''));');

%*****Color axis*****
text_caxis = uicontrol(fgui,...
  'Style','text',...
  'String','ATC colorbar:',...
  'HorizontalAlignment','Left',...
  'Position',[(plot1+10) (plot4-240) (plot3-140) 20]);
en_caxis = uicontrol(fgui,...
  'Style','edit',...
  'Pos',[(plot1+110) (plot4-235) 110 20],...
  'String',num2str(accumthetacaxis),...
  'CallBack','accumthetacaxis = str2num(get(en_caxis,''String''));');



%*****Data query button commands*****
pb_query = uicontrol(fgui,...
  'Style','push',...
  'String','Data query',...
  'Pos',[plot1+10 20 100 30],...
  'CallBack',['query=1;,','cl_bblqry']);
%*****Compute/Plot button commands*****
pb_plot = uicontrol(fgui,...
  'Style','push',...
  'String','Compute/Plot',...
  'Pos',[plot1+120 20 100 30],...
  'CallBack','cl_bblplt');




